// Vertex Shader – file "minimal.vert"

#version 130

//IN VERTEX ATTRIBUTES
in  vec3 in_Position; // incoming vertex position
in  vec3 in_Color; // incoming vertex color
in	vec3 in_Normal;

//OUT VERTEX ATTRIBUTES
out vec3 ex_Color;

//UNIFORM TRANSFORMATION PASSING
uniform vec3 in_Translation;
uniform vec3 in_Scale;
uniform vec3 rot_axis_X;
uniform float rot_angle_X;
uniform vec3 rot_axis_Y;
uniform float rot_angle_Y;

//UNIFORM MATRIX PASSING - NOT USED AT THE MOMENT
uniform mat4 modelview_matrix; // object space to CAMEREA SPACE transformation
uniform mat4 projection_matrix; //projective transformation

//LOCAL VARIABLES
const vec3 lightPos = vec3(20.0,20.0,-20.0);
const float specContr = 0.3;
const float diffContr = 1.0 - specContr;

const vec3 lightPosG = vec3(10.0,10.0,40.0);
const float specContrG = 0.6;
const float diffContrG = 1.0 - specContr;

const int MODELVIEW = 0;
const int PROJECTION = 1;

int current_mode = MODELVIEW;
mat4 current_matrix = mat4(1.0);
mat4 MV_matrix = mat4(1.0);
mat4 PROJ_matrix = mat4(1.0);

//////////////////////////////////////
//PROJECTION MATRIX CALCULATOR		//
//////////////////////////////////////
mat4 aplProjectionMatrix(const in float left, const in float right,
                         const in float bottom, const in float top,
                         const in float zNear, const in float zFar){

	vec3 delta	= vec3(right, top, zFar) - vec3(left, bottom, zNear);
	vec3 sum	= vec3(right, top, zFar) + vec3(left, bottom, zNear);
	vec3 ratio	= sum / delta;
	vec3 twoRatio = 2.0 * zNear / delta;
	return mat4(twoRatio.x,		0.0,			0.0,			0.0, 
				0.0,			twoRatio.y,		0.0,			0.0, 
				ratio.x,		ratio.y,		-ratio.z,		-1.0,
				0.0,			0.0,			-zNear*twoRatio.z,		0.0);
}
//////////////////////////////////////
//TRANSFORMATION MATRIX CALCULATOR	//
//////////////////////////////////////
mat4 aplIdentityMatrix(){
	return mat4(1.0, 0.0, 0.0, 0.0,
				0.0, 1.0, 0.0, 0.0,
				0.0, 0.0, 1.0, 0.0,
				0.0, 0.0, 0.0, 1.0);
}

mat4 aplTranslationsMatrix(const in vec3 translate){
	return mat4(1.0,		0.0,		0.0,			0.0, 
				0.0,		1.0,		0.0,			0.0, 
				0.0,		0.0,		1.0,			0.0,
				translate.x, translate.y, translate.z, 1.0);
}

mat4 aplScaleMatrix(const in vec3 scale){
	return mat4(scale.x,	0.0,		0.0,		0.0,
				0.0,		scale.y,	0.0,		0.0,
				0.0,		0.0,		scale.z,	 0.0,
				0.0,		0.0,		0.0,		1.0);
	
}

mat4 aplRotateMatrix(const in float angle, const in vec3 axis){
	vec3 n = normalize(axis);
	float theta = radians(angle);
	float c = cos(theta);
	float s = sin(theta);
	mat3 R;
	R[0] = n.xyz*n.x*(1.0-c) + vec3( c,		 n.z*s,	-n.y*s);
	R[1] = n.xyz*n.y*(1.0-c) + vec3(-n.z*s,	 c,		 n.x*s);
	R[2] = n.xyz*n.z*(1.0-c) + vec3( n.y*s,	-n.x*s,		 c);
	return mat4(R[0],			0.0,
				R[1],			0.0,
				R[2],			0.0,
				0.0, 0.0, 0.0,  1.0);

}

mat4 aplRotateMatrixTwoAxes(const in float angle1, const in vec3 axis1,
							const in float angle2, const in vec3 axis2){
	return aplRotateMatrix(angle1, axis1)*aplRotateMatrix(angle2, axis2);
}

//////////////////////////////////////////////////////////////////
//MULTIPLYING TRANSFORMATION  AND PROJECTION WITH CURRENT MATRIX//
//////////////////////////////////////////////////////////////////
void aplMatrixMode(const in int mode){
	if (mode != current_mode){
		switch (mode){
			case MODELVIEW:
			{
				PROJ_matrix = current_matrix;
				current_matrix = MV_matrix;
				current_mode = MODELVIEW;
				break;
			}
			case PROJECTION:
			{
				MV_matrix = current_matrix;
				current_matrix = PROJ_matrix;
				current_mode = PROJECTION;
				break;
			}
			default:
			{
				break;
			}
		}
	}
	return;
}

void aplLoadIdentity(void){
	current_matrix = aplIdentityMatrix();
}

void aplLoadMatrix(const in mat4 m){
	current_matrix = m;
}

void aplMultMatrix(const in mat4 m){
	current_matrix = m * current_matrix;
}

void aplScale(const in vec3 s){
	current_matrix = aplScaleMatrix(s)*current_matrix;
}

void aplTranslate(const in vec3 t){
	current_matrix = aplTranslationsMatrix(t)*current_matrix;
}

void aplRotate(const in float angle, const in vec3 axis){
	current_matrix = aplRotateMatrix(angle, axis)*current_matrix;
}

void aplFrustum(const in float left, const in float right,
                const in float bottom, const in float top,
                const in float zNear, const in float zFar){
	               
	current_matrix = aplProjectionMatrix(left, right, bottom, top,
										 zNear, zFar)*current_matrix;        
 }

//////////////////////////////////////
//MAIN								//
//////////////////////////////////////
void main(void)
{	
	aplMatrixMode(PROJECTION);
	aplLoadIdentity();
	aplFrustum(-8, 8, -6, 6, 20, 500);
	
	aplMatrixMode(MODELVIEW);//swaps matrices!
	aplLoadIdentity();
	aplScale(in_Scale);
	aplRotate(rot_angle_Y, rot_axis_Y);
	aplRotate(rot_angle_X, rot_axis_X);
	aplTranslate(in_Translation);
	aplMatrixMode(PROJECTION);//swaps matrices!
	
	vec3 ec_Position = vec3(MV_matrix * vec4(in_Position,1.0));
	vec4 ecv_Position = vec4(ec_Position, 1.0);
	gl_Position = vec4(PROJ_matrix * ecv_Position);
	
	//LIGHT INTENSITY CALCULATION
	
	vec3 normVec = normalize(in_Normal);
	vec3 viewVec = normalize(in_Position);
	
	//first light source (YELLOW)
	
	vec3 lightVec = normalize(lightPos - in_Position);
	
	vec3 reflVec = reflect(-lightVec,normVec);
	reflVec = normalize((PROJ_matrix *MV_matrix*vec4(reflVec, 0.0)).xyz);
	
	float diffuse = max(dot(lightVec,normVec),0.0);
	float spec = 0.0;
	if (diffuse > 0.0){
		spec = max(dot(reflVec,viewVec),0.0);
		spec = pow(spec, 32.0);//sharpen specular part
	}
	
	//second light source (GREEN)
	
	vec3 lightVecG = normalize(lightPosG - in_Position);
	vec3 reflVecG = reflect(-lightVecG,normVec);
	reflVecG = normalize((PROJ_matrix *MV_matrix*vec4(reflVecG, 0.0)).xyz);
	
	float diffuseG = max(dot(lightVecG,normVec),0.0);
	float specG = 0.0;
	if (diffuseG > 0.0){
		specG = max(dot(reflVecG,viewVec),0.0);
		specG = pow(specG, 32.0);//sharpen specular part
	}
	
	//ambient + yellow + green
	float lightIntRED = 0.3 + (diffContr*diffuse + specContr*spec) + 1.0*(diffContrG*diffuseG + specContrG*specG); //0.59
	float lightIntGREEN = 0.3 + 1.0*(diffContr*diffuse + specContr*spec) + (diffContrG*diffuseG + specContrG*specG); //0.706
	float lightIntBLUE = 0.3;
	
	ex_Color.r = lightIntRED * in_Color.r;//RED
	ex_Color.g = lightIntGREEN * in_Color.g;//GREEN
	ex_Color.b = lightIntBLUE * in_Color.b;//BLUE

}
